<?php

namespace Luticate\Utils;

abstract class LuBusiness {

    /**
     * @param string $name
     * @param mixed  $default
     * @return mixed
     */
    public static function getParam($name, $default = null)
    {
        return array_key_exists($name, LuController::$parameters) ? LuController::$parameters[$name] : $default;
    }

    /**
     * @param string[] $params
     * @return bool
     */
    public static function hasParam(array $params)
    {
        foreach ($params as $p) {
            if (!array_key_exists($p, LuController::$parameters)) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param string $param
     * @param mixed $validate
     * @return string
     */
    public static function checkParam($param, $validate = null)
    {
        $value = self::getParam($param);
        if (is_null($value) || trim($value) == '')
            abort(400, 'Missing parameter: ' . $param);
        if ($validate != null) {
            if (is_callable($validate)) {
                $validate = $validate($value);
            }
            else {
                $validate = preg_match($validate, $value);
            }
            if (!$validate)
                abort(400, 'Bad parameter value: ' . $param);
        }
        return $value;
    }

    /**
     * @param string
     */
    public static function unauthorized($reason = 'Unauthorized')
    {
        abort(401, $reason);
    }

    /**
     * @param string
     */
    public static function notFound($reason = 'Resource not found')
    {
        abort(404, $reason);
    }

    /**
     * @param string
     */
    public static function badInput($reason = 'Invalid user input')
    {
        abort(400, $reason);
    }

    /**
     * @param $param string
     * @param $values string[]
     * @return string
     */
    public static function getParamInArray($param, $values)
    {
        $value = self::getParam($param);
        if (is_null($value) || !in_array($value, $values))
            $value = $values[0];
        return $value;
    }

    /**
     * @param $param string
     * @param $default int
     * @return int
     */
    public static function getParamInt($param, $default = 0)
    {
        $value = self::getParam($param);
        if (!is_null($value) && is_numeric($value))
            return (int)$value;
        return $default;
    }

    /**
     * @param $page int
     * @param $perPage int
     * @param $column string
     * @param $order string
     * @param $query string
     * @return LuDbo[]
     */
    public static function search($page, $perPage, $column, $order, $query = null)
    {
        return static::getDataAccess()->search($page, $perPage, $column, $order, $query);
    }

    /**
     * @param $id int
     * @return LuDbo|null
     */
    public static function getById($id)
    {
        $data = static::getDataAccess()->getById($id);
        if (is_null($data))
            self::notFound();
        return $data;
    }

    /**
     * @param $id int
     */
    public static function deleteById($id)
    {
        static::getDataAccess()->deleteById($id);
    }

    /**
     * @return LuDataAccess
     */
    protected static function getDataAccess()
    {
        return null;
    }
}
